/***************************************************************************
 *  Copyright:http://www.naradabrokering.org/index.html
 *  Author: Jaliya Ekanayake
 *  Email : jaliyae@gmail.com
 ****************************************************************************/
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include "AgentConfig.h"
#include "ServiceException.h"


using namespace std;

AgentConfig::~AgentConfig(){}

AgentConfig::AgentConfig(string configFile):configFile(configFile){}

/*
 * Populate the AgentConfig class from the given configuration file.
 * */
void
AgentConfig::getConfiguration(ClientContext *ctx)
{
	 ifstream file(configFile.c_str());
	 
	 if( file.fail() ){
	 	throw ServiceException("Could not locate the config file");
	 }
	 
	 string line;
	 while (getline(file,line)) {
	 	 if (! line.length()) continue;
	 	 
	 	 if(line.length()<3) continue;
	 	 
	 	 if(line[0]=='n' && line[1]=='b' && line[2]=='_')
	 	 {
	 	 	int pos=line.find_first_of("=",0);
	 	 	if(pos==(signed int)string::npos) 
	 	 	{
	 	 		throw ServiceException("Error in client_config.txt");
	 	 	}
	 	 	else
	 	 	{
	 	 		string varName=trim(line.substr(0,pos));
	 	 		string varValue=trim(line.substr(pos+1));/*Exclude = mark*/
	 	 		
	 	 		if(varName==CONF_NB_HOST_NAME) ctx->setBrokerHost(varValue);
	 	 		else if(varName==CONF_NB_PORT_NUMBER) ctx->setBrokerPort(atoi(varValue.c_str()));
	 	 		else if(varName==CONF_NB_TRANSPORT) ctx->setBrokerTransport(varValue);
	 	 		else if(varName==CONF_NB_ENTITY_ID) ctx->setEntityId(atoi(varValue.c_str()));	 	 		
	 	 		else if(varName==CONF_NB_KEYSTORE) ctx->setKeystoreFile(varValue);
	 	 		else if(varName==CONF_NB_KEYSTORE_PWD) ctx->setKeystorePwd(varValue);
	 	 		else if(varName==CONF_NB_KEY_ALIAS) ctx->setKeyAlias(varValue);
	 	 		else if(varName==CONF_NB_PRIVATE_KEY_PWD) ctx->setPrivateKeyPwd(varValue);
	 	 		else if(varName==CONF_NB_TRUSTED_CAS) ctx->setTrustedCAs(varValue);
	 	 		else if(varName==CONF_NB_PROXY_FILE) ctx->setProxyFile(varValue);
	 	 	}
	 	 }
	 }
}

string 
AgentConfig::trim(string const& source) {	
	char const* delims = " \t\r\n";
  string result(source);
  string::size_type index = result.find_last_not_of(delims);
  if(index != string::npos)
    result.erase(++index);

  index = result.find_first_not_of(delims);
  if(index != string::npos)
    result.erase(0, index);
  else
    result.erase();
  return result;
}


