/***************************************************************************
 *  Copyright:http://www.naradabrokering.org/index.html
 *  Author: Jaliya Ekanayake
 *  Email : jaliyae@gmail.com
 ****************************************************************************/
#include "ClientCall.h"
#include "ServiceException.h"
#include "Utils.h"

/*
 * Populates the arrary of char pointers using the
 * response messages. Caller should deserialize the message
 * according to its type.
 * */

class ClientCallCallback : public Callback
{
	private:
		int *numMsgs;
		char **res;
	
	public:
		
	ClientCallCallback(int *numMsgs, char** res):numMsgs(numMsgs),res(res){}
		
	void onEvent(NBEvent *nbEvent){
		(*numMsgs)++; /*Increment the message count*/
				
		cout<<endl<<"Event Rececived with for the topic - "<<nbEvent->getContentSynopsis()<<endl;
		/*
		 * We create a new char array in the heap for the payload. 
		 * So we can pass this pointer to the client. It should be deleted at the caller.
		 * */
		res[(*numMsgs)-1]=nbEvent->getContentPayload();
		//int payloadSize=nbEvent->getPayloadLength();		
	}
};


ClientCall::ClientCall(){}
ClientCall::~ClientCall(){}

/*
 * Initialize the ServiceClient used for the call.
 * */
void 
ClientCall::init(ClientContext *ctx)
{
	/*TODO resolve the templateId */	
	try{
		this->serviceClient.init(ctx->getBrokerHost(),ctx->getBrokerPort(),ctx->getEntityId(),4321);
	}catch(ServiceException& e){
		throw e;
	}
}

/*
 * Blocking invocation based on the underlying publish/subscribe API.
 * This will wait for maximum of CLIENT_CALL_MAX_WAIT to receive a 
 * response or responses specified by noOfResponse
 * Number of received responses is passed back using numReturn value.
 * 
 * */
bool
ClientCall::sendReceive(string pubTopic, string subTopic, char* data,int size_of_data,int noOfResponse,char** responses,int *numReturn)
{	
	int numMsg=0;		
	
	ClientCallCallback ccCallback(&numMsg,responses);
	this->serviceClient.subscribe(subTopic,&ccCallback);	
	
    this->serviceClient.publish(pubTopic,data,size_of_data); 	
	
	long long start=Utils::getCurrentTimeMicroseconds();
	while(true)
	{
		if(numMsg>=noOfResponse)
		{			
			(*numReturn)=numMsg;
			break;
		}
		else 
		{
			long long end=Utils::getCurrentTimeMicroseconds();
			if((end-start)>CLIENT_CALL_MAX_WAIT)
			{	
				break;
			}
		}		
		usleep(CLIENT_CALL_SLEEP);
	}
	

	/*
	 * Should unsubscribe from the topics. 
	 * */
	this->serviceClient.unsubscribe(subTopic);	
	
	return false;
}

void 
ClientCall::send(string pubTopic, char* data)
{
	
}

void ClientCall::close() 
{
	
}
