/***************************************************************************
 *  Copyright:http://www.naradabrokering.org/index.html
 *  Author: Jaliya Ekanayake
 *  Email : jaliyae@gmail.com
 ****************************************************************************/
#include "MessageCreator.h"
#include "AgentDiscoveryRequest.h"
#include "ServiceDiscoveryRequest.h"
#include "Utils.h"
#include "NBCCoverMessage.h"
#include "constants.h"
#include "ServiceException.h"
#include "EncryptedCoverMessage.h"

#include "assert.h"


MessageCreator::MessageCreator(){}
MessageCreator::~MessageCreator(){}

void
MessageCreator::init(ClientContext *ctx,UserRSA *userRSA,  SecurityUtil *secUtil){
	this->ctx=ctx;
	this->userRSA=userRSA;
	this->secUtil=secUtil;
}

void 
MessageCreator::createDiscoverAgentRequest(string subTopic,NBCEnvelope *env){
	
	string reqId =Utils::getCurrentTimeMicroAsString();
	string actionId = Utils::getCurrentTimeMicroAsString();

	AgentDiscoveryRequest payloadMsg;
	payloadMsg.setRequestId(reqId);

	NBCCoverMessage coverMsg;
	coverMsg.setDataType(PROTOCOL_MESSAGE);
	coverMsg.setActionId(actionId);
	coverMsg.setActionType(REQUEST);
	coverMsg.setExchangeType(AGENT_DISCOVERY);
	coverMsg.setCommTopic(subTopic);
	coverMsg.setPayloadMessage(&payloadMsg);
	
	/*
	 * These bytes will be deleted at the distructor of the NBCEnvelope
	 * as it deletes the payload, signature and the certificate bytes.
	 * */
	char *payload_bytes=new char[coverMsg.getSerializedLength()];
	int payload_size=0;
	coverMsg.getBytes(payload_bytes,&payload_size);	
	if(coverMsg.getSerializedLength()<=payload_size){
		cout<<"ERROR : Over-written for NBCCoverMessage"<<endl;
		exit(-1);
	}
	


	int sig_size=0;
	char * sig_buff=NULL;
	try{
		sig_buff=this->secUtil->signBytes(payload_bytes,(unsigned int)payload_size,(RSA*)userRSA->getPrivateRSA(),&sig_size);
	}catch(SecurityException& e){
		throw  ServiceException(e.getMessage());
	}
		
	//cout<<"WRITING THE ENVELOPE payload size = "<<payload_size<<" signature size "<<sig_size<<endl;
	
	env->setSecure(false);
	env->setPayload(payload_bytes,payload_size);
	env->setSignature((char*)sig_buff,sig_size);
	env->setCertString(userRSA->getUserCert(),userRSA->getUserCert().size());		
	
}

void 
MessageCreator::createDiscoverServiceRequest(string subTopic,NBCEnvelope *env,X509 *cert){
	
	cout<<"Coming to MessageCreator for Service Discovery 0000000000000000"<<endl;
	
	//=================================
	//Get the public key from user certificate
    EVP_PKEY * user_pub_key = X509_get_pubkey (cert);
    X509_free (cert);
    if (!user_pub_key)
    { 
      cout<<"Could not read user public key:\n";                  
    }
    
    //Get the user Public RSA from the public key
    RSA*   m_userPublicRSA = EVP_PKEY_get1_RSA (user_pub_key);
    EVP_PKEY_free (user_pub_key);
    if (!m_userPublicRSA)
    { 
      cout<<"Could not extract RSA structure from user public key:\n";        
    }
	
	
	//===================================
	
	
	
	
	string reqId =Utils::getCurrentTimeMicroAsString();
	string actionId = Utils::getCurrentTimeMicroAsString();

	ServiceDiscoveryRequest payloadMsg;
	payloadMsg.setRequestId(reqId);

	NBCCoverMessage coverMsg;
	coverMsg.setDataType(PROTOCOL_MESSAGE);
	coverMsg.setActionId(actionId);
	coverMsg.setActionType(REQUEST);
	coverMsg.setExchangeType(ROOTLET_DISCOVERY);
	coverMsg.setCommTopic(subTopic);
	coverMsg.setPayloadMessage(&payloadMsg);
	
	
	
	char msg_bytes[coverMsg.getSerializedLength()];
	int msg_size=0;
	coverMsg.getBytes(msg_bytes,&msg_size);
	/*Scanity Check*/	
	assert(coverMsg.getSerializedLength()>=msg_size);
	
	/*Generate a secret key*/
	string secKey=this->secUtil->generateAESKey();
	
	/*Allocate a buffer*/
	unsigned char encbuf[AES_INC+msg_size];
	bzero (& encbuf, AES_INC+msg_size);	
	int outLength=0;
	/*Encrypt the message*/		
	if(!secUtil->encryptAES((const unsigned char*)msg_bytes,msg_size,encbuf,outLength,secKey)){
		cout<<"Encryption Faild"<<endl;	
	}
	
	///RSA * publicRSA=(RSA *)this->userRSA->getPublicRSA();
	cout<<"RSA_size(publicRSA) = "<< RSA_size(m_userPublicRSA)<<endl;
	unsigned char rsabuf[RSA_size(m_userPublicRSA)];
	bzero (& rsabuf, RSA_size(m_userPublicRSA));	
	
	int encLength=RSA_public_encrypt(secKey.length(),(unsigned char *)secKey.c_str(),rsabuf,m_userPublicRSA,RSA_PKCS1_PADDING);
	cout<<"Encrypted Length = "<<encLength<<endl;
		

		
	//Set the message + secret key here.	
	EncryptedCoverMessage encCoverMsg;
	cout<<"Populating EncryptedCoverMessage, msgLength= "<<outLength<<"  encrypted key length ="<<encLength<<endl;
	encCoverMsg.setMessage((char *)encbuf,outLength);
	encCoverMsg.setSecretKey((char *)rsabuf,encLength);
		
	/*
	 * These bytes will be deleted at the distructor of the NBCEnvelope
	 * as it deletes the payload, signature and the certificate bytes.
	 * */
	 
	cout<<"encCoverMsg.getSerializedLength() = "<<encCoverMsg.getSerializedLength()<<endl;
	char *payload_bytes=new char[encCoverMsg.getSerializedLength()];
	
	int payload_size=0;
	encCoverMsg.getBytes(payload_bytes,&payload_size);
	/*Scanity Check*/	
	assert(encCoverMsg.getSerializedLength()>=payload_size);
	

	
	int sig_size=0;
	char * sig_buff=NULL;
	try{
		sig_buff=this->secUtil->signBytes(payload_bytes,(unsigned int)payload_size,(RSA*)userRSA->getPrivateRSA(),&sig_size);
	}catch(SecurityException& e){
		throw  ServiceException(e.getMessage());
	}
		
	//cout<<"WRITING THE ENVELOPE payload size = "<<payload_size<<" signature size "<<sig_size<<endl;
	
	env->setSecure(true);
	env->setPayload(payload_bytes,payload_size);
	env->setSignature((char*)sig_buff,sig_size);
	env->setCertString(userRSA->getUserCert(),userRSA->getUserCert().size());		
	

}

