/***************************************************************************
 *  Copyright:http://www.naradabrokering.org/index.html
 *  Author: Jaliya Ekanayake
 *        : Conrad Steenberg
 *  Email : jaliyae@gmail.com
 ****************************************************************************/
#include <iostream>
#include <fstream>
#include "ServiceClient.h"
#include "Profile.h"

// Added for unix pipes
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/errno.h>
#include <fcntl.h>


using namespace std;

class MyCallback : public Callback {
    void onEvent(NBEvent *nbEvent) {
      string eventstring(nbEvent->getContentPayload(),
                         nbEvent->getPayloadLength());
      cout<<"Received message "<<eventstring<<endl;
    }
  };


int main(int argc, char **argv) {
  cout<<"///////////////////////TestClient////////////////////////"<<endl;

  if (argc!=7) {
      cout<<"Usage : [entityId][templateId][topic][host][port]"<<endl;
      cout<<"E.g. 2366 88854 /topic/test 156.56.104.200 5045 "<<endl;
      exit(1);
    }
  /*Ignore the first argument */
  *argv++;
  int entityId=0;
  istringstream ss1(*argv++);
  ss1 >> entityId;

  int templateId=0;
  istringstream ss2(*argv++);
  ss2 >> templateId;

  string contentSynopsis=*argv++;

  string host=*argv++;

  int port=0;
  istringstream ss3(*argv++);
  ss3 >> port;

  if (entityId<=0|| contentSynopsis.length()<=0) {
      cout<<"Usage : [entityId][templateId][topic][host][port]"<<endl;
      cout<<"E.g. 2366 88854 /topic/test 156.56.104.200 5045 "<<endl;
      exit(1);
    }

  // Add pipe handling
  string pipename=*argv++;
  int pipefd=mkfifo(pipename.c_str(),S_IRUSR|S_IWUSR);
  if (pipefd==-1){
    string err_msg("Error creating FIFO ");
    err_msg+=pipename;
    switch (errno){
      case (EEXIST): err_msg+="File exists";
        break;
      case (EACCES): err_msg+="Access denied";
        break;
      case (ENAMETOOLONG): err_msg+="Name too long";
        break;
      case (ENOENT): err_msg+="Directory component doesn't exist, or dangling symlink";
        break;
      case (ENOSPC): err_msg+="No space left on file system";
        break;
      case (ENOTDIR): err_msg+="Part of pathname is not a directory";
        break;
      case (EROFS): err_msg+="Read-only file system";
        break;
      default: break;
    }
    if (errno!=EEXIST){
      cerr<<err_msg<<", exiting"<<endl;    
      exit(errno);
    }
    cerr<<err_msg<<endl;      
  }
  else close(pipefd);

  FILE *pipestream;
  ServiceClient serviceClient;
  serviceClient.init(host,port,entityId,templateId);

  MyCallback callback;
  serviceClient.subscribe(contentSynopsis,&callback);
  string str="";
  char *msg;

  while (1){
    pipefd=open(pipename.c_str(),O_RDONLY);
    if (pipefd==-1) {
      cerr<<"Opening file "<<pipename<<" failed!"<<endl;
      exit(0);
    }
    else cerr<<"Opening file "<<pipename<<" succeeded.."<<endl;


    pipestream=fdopen(pipefd,"r");
    cerr<<pipename<<" opened successfully: fd="<<pipefd<<endl;


    while (str!=".") {
        cout<<"Waiting for input.."<<endl;

        size_t n;
        ssize_t bytes_read;
        msg=NULL;
        bytes_read=getline(&msg, &n, pipestream);

        if (bytes_read==-1){
          cerr<<"Error reading from pipe "<<pipename<<". Clearing..."<<endl;
          fclose(pipestream);
          close(pipefd);
          break;
        }

        if (bytes_read>1 & msg[0]!='.') {
            serviceClient.publish(contentSynopsis,msg,bytes_read-1);
            msg[bytes_read-1]='\0';
            cout<<"published message: '"<<msg<<"'"<<endl;
            if (msg) free(msg);
          }
        else {
          if (msg) free(msg);
          goto close_tag;
        }
    }
  }

close_tag:  serviceClient.cleanup();

}
