/***************************************************************************
 *  Copyright:http://www.naradabrokering.org/index.html
 *  Author: Jaliya Ekanayake
 *  Email : jaliyae@gmail.com
 ****************************************************************************/
#include "SecurityUtil.h"
#include "Utils.h"

SecurityUtil::SecurityUtil(){}

SecurityUtil::~SecurityUtil(){}


void 
SecurityUtil::init(){
	
	EVP_MD_CTX_init(&vctx);	
}

bool 
SecurityUtil::verifySignature(NBCEnvelope &env){
	
		EVP_VerifyInit_ex(&vctx, EVP_sha1(), NULL);
		
		X509 *cert;
		env.getCertificate(&cert);			
		
		EVP_PKEY * pub_key = X509_get_pubkey (cert);   		 		
   		X509_free (cert);
    	if (!pub_key)
   		 { 
			cerr<<"ERROR: Reading the agent's certificate to get the pubkey"<<endl;
			return false;
   		 }
   		
		
		EVP_VerifyUpdate(&vctx,env.getPayload() ,env.getPayloadLength());
		
		int ret = EVP_VerifyFinal(&vctx, (unsigned char *)env.getSignature(), env.getSignatureLength(), pub_key);
        if (ret == 1)
        {
        	   std::cout << "Signature is Valid..." << std::endl;
               return true;    
                                
        }
        else if (ret == 0){
                std::cout << "Signature is Invalid..." << std::endl;
                return false;
        }
        else{
                std::cout << "Verification Failed..." << std::endl;
                return false;
        }		
 }
 
char *
SecurityUtil::signBytes(char * bytes,unsigned int size, RSA  *privateRSA, int *s_size ){

	EVP_SignInit_ex(&vctx,EVP_sha1(), NULL);
	EVP_SignUpdate(&vctx,bytes,size);
	
	EVP_PKEY *pkey = NULL;
	
	if (!(pkey = EVP_PKEY_new())) {
		throw  SecurityException("Could not create EVP key");
	}
	
	if (!EVP_PKEY_assign_RSA(pkey, privateRSA)) {		
		EVP_PKEY_free(pkey);
		throw  SecurityException("Could not conver the RSA to  EVP key ");
	}
	
	/*
	 * These bytes will be deleted at the distructor of the NBCEnvelope.
	 * */
	unsigned char *sig_buff=new unsigned char[EVP_PKEY_size(pkey)];
	unsigned int sig_size;	
	
	if (!EVP_SignFinal(&vctx,sig_buff,&sig_size,pkey))
	{
		EVP_PKEY_free(pkey);
		delete [] sig_buff;
		throw  SecurityException("Signing failed ");
	}
	
	
	if(EVP_PKEY_size(pkey)<(signed int)sig_size){
		//TODO Remove this
		//something terribly wrong.		
		cerr<<"ERROR : Over-written for signature EVP_PKEY_size(pkey)"<< EVP_PKEY_size(pkey)<< " sig_size = "<<sig_size<<endl;
		exit(-1);
	}
	
	(*s_size)=sig_size;
	return (char*)sig_buff;
 
}

string 
SecurityUtil::generateAESKey(){
	//TODO Fix this to generate 128 bit AES keys
	//Right now we just use the current time in microseconds as the key
	
	return Utils::getCurrentTimeMicroAsString();
}


bool 
SecurityUtil::encryptAES(const unsigned char* in,int inputLength ,unsigned char *out,int &outputLength, string aesKey){
	
	int olen, tlen;
	EVP_CIPHER_CTX ctx;
	EVP_CIPHER_CTX_init (& ctx);
	EVP_EncryptInit (& ctx, EVP_aes_128_ecb (), (unsigned char *)aesKey.c_str(), NULL);
		
	
	if (EVP_EncryptUpdate (& ctx, out, & olen, (const unsigned char*)in , inputLength) != 1)
	{
	   cerr<<"error in decrypt update"<<endl;
	   return false;
	}

	if (EVP_EncryptFinal (& ctx, out + olen, & tlen) != 1)
	{
	   cerr<<"error in encrypt final"<<endl;
	   return false;
	}
	
	olen+=tlen;
	
	outputLength=olen;
	
	EVP_CIPHER_CTX_cleanup (& ctx);
	return true;

}

bool 
SecurityUtil::decryptAES(const unsigned char *in,int inputLength ,unsigned char *out,int &outputLength, string aesKey){
	int olen, tlen;
	EVP_CIPHER_CTX ctx;
	EVP_CIPHER_CTX_init (& ctx);
	EVP_DecryptInit (& ctx, EVP_aes_128_ecb (), (unsigned char *)aesKey.c_str(), NULL);
	
	olen=0; tlen=0;
	
	if (EVP_DecryptUpdate (& ctx, out, & olen, (const unsigned char*)in,inputLength) != 1)
	{
	    cerr<<"error in decrypt update"<<endl;
	    return false;
	}
	
	if (EVP_DecryptFinal(& ctx, out + olen, & tlen) != 1)
	{
		cerr<<"error in decrypt final"<<endl;
		return false;
	}
	olen += tlen;
	
	outputLength=olen;
	
	EVP_CIPHER_CTX_cleanup (& ctx);
	return true;
}

